@@ -55,18 +55,20 @@ void   _secFreeAccount(struct oidc_account* p);
 void   secFreeAccountContent(struct oidc_account* p);
 
 struct oidc_account* updateAccountWithPublicClientInfo(struct oidc_account*);
+struct oidc_account* updateAccountWithUserClientInfo(struct oidc_account*);
 char*                getScopesForPublicClient(const struct oidc_account*);
+char*                getScopesForUserClient(const struct oidc_account*);
 int                  accountConfigExists(const char* accountname);
 char*                getAccountNameList(list_t* accounts);
 int                  hasRedirectUris(const struct oidc_account* account);
 
-int  account_matchByState(const struct oidc_account* p1,
+int   account_matchByState(const struct oidc_account* p1,
+                           const struct oidc_account* p2);
+int   account_matchByName(const struct oidc_account* p1,
                           const struct oidc_account* p2);
-int  account_matchByName(const struct oidc_account* p1,
-                         const struct oidc_account* p2);
-int  account_matchByIssuerUrl(const struct oidc_account* p1,
-                              const struct oidc_account* p2);
-void account_setOSDefaultCertPath(struct oidc_account* account);
+int   account_matchByIssuerUrl(const struct oidc_account* p1,
+                               const struct oidc_account* p2);
+char* getDefaultCertPath();
 
 // make setters and getters avialable
 #include "account/setandget.h"
