@@ -1,11 +1,12 @@
 #include "oidc-agent_options.h"
 
 #include "utils/agentLogger.h"
+#include "utils/config/agent_config.h"
+#include "utils/listUtils.h"
 #include "utils/string/stringUtils.h"
 
 #define OPT_NOAUTOLOAD 2
 #define OPT_NO_WEBSERVER 3
-#define OPT_PW_STORE 4
 #define OPT_GROUP 5
 #define OPT_NO_SCHEME 6
 #define OPT_LOG_CONSOLE 7
@@ -16,24 +17,24 @@
 #define OPT_NO_AUTOREAUTHENTICATE 12
 
 void initArguments(struct arguments* arguments) {
-  arguments->kill_flag               = 0;
-  arguments->console                 = 0;
-  arguments->debug                   = 0;
-  arguments->lifetime                = 0;
-  arguments->no_autoload             = 0;
-  arguments->confirm                 = 0;
-  arguments->no_webserver            = 0;
-  arguments->pw_lifetime.lifetime    = 0;
-  arguments->pw_lifetime.argProvided = 0;
-  arguments->group                   = NULL;
-  arguments->socket_path             = NULL;
-  arguments->no_scheme               = 0;
-  arguments->always_allow_idtoken    = 0;
-  arguments->log_console             = 0;
-  arguments->status                  = 0;
-  arguments->json                    = 0;
-  arguments->quiet                   = 0;
-  arguments->no_autoreauthenticate   = 0;
+  arguments->kill_flag             = 0;
+  arguments->console               = 0;
+  arguments->debug                 = getAgentConfig()->debug;
+  arguments->lifetime              = getAgentConfig()->lifetime;
+  arguments->no_autoload           = !getAgentConfig()->autoload;
+  arguments->confirm               = getAgentConfig()->confirm;
+  arguments->no_webserver          = !getAgentConfig()->webserver;
+  arguments->group                 = getAgentConfig()->group;
+  arguments->socket_path           = getAgentConfig()->bind_address;
+  arguments->no_scheme             = !getAgentConfig()->customurischeme;
+  arguments->always_allow_idtoken  = getAgentConfig()->alwaysallowidtoken;
+  arguments->log_console           = 0;
+  arguments->status                = 0;
+  arguments->json                  = 0;
+  arguments->quiet                 = 0;
+  arguments->no_autoreauthenticate = !getAgentConfig()->autoreauth;
+  arguments->command               = NULL;
+  arguments->args_list             = NULL;
 }
 
 static struct argp_option options[] = {
@@ -72,11 +73,6 @@ static struct argp_option options[] = {
      "authorization code flow is used. oidc-agent will not use a custom uri "
      "scheme redirect.",
      1},
-    {"pw-store", OPT_PW_STORE, "TIME", OPTION_ARG_OPTIONAL,
-     "Keeps the encryption passwords for all loaded account configurations "
-     "encrypted in memory for TIME seconds. Can be overwritten for a specific "
-     "configuration with oidc-add. Default value for TIME: Forever",
-     1},
     {"with-group", OPT_GROUP, "GROUP_NAME", OPTION_ARG_OPTIONAL,
      "This option allows that applications running under another user can "
      "access the agent. The user running the other application and the user "
@@ -137,17 +133,22 @@ static error_t parse_opt(int key, char* arg __attribute__((unused)),
       }
       arguments->lifetime = strToInt(arg);
       break;
-    case OPT_PW_STORE:
-      arguments->pw_lifetime.argProvided = 1;
-      arguments->pw_lifetime.lifetime    = strToULong(arg);
-      break;
     case OPT_JSON: arguments->json = 1; break;
     case OPT_QUIET: arguments->quiet = 1; break;
     case OPT_NO_AUTOREAUTHENTICATE: arguments->no_autoreauthenticate = 1; break;
     case 'h':
       argp_state_help(state, state->out_stream, ARGP_HELP_STD_HELP);
       break;
-    case ARGP_KEY_ARG: argp_usage(state); break;
+    case ARGP_KEY_ARG:
+      if (state->arg_num == 0) {
+        arguments->command   = arg;
+        arguments->args_list = list_new();
+      }
+      list_rpush(arguments->args_list, list_node_new(arg));
+      break;
+    case ARGP_KEY_END:
+      arguments->args = (char* const*)listToArray(arguments->args_list);
+      break;
     default: return ARGP_ERR_UNKNOWN;
   }
   return 0;
