@@ -4,6 +4,5 @@
 #include "utils/lifetimeArg.h"
 #include "utils/oidc_error.h"
 
-oidc_error_t pw_handleSave(const char*              pw_entry_str,
-                           const struct lifetimeArg pw_lifetime);
+oidc_error_t pw_handleSave(const char* pw_entry_str);
 #endif  // OIDC_PASSWORD_HANDLER_H
